from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.component_classifier import ComponentClassifier
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.component_property import ComponentProperty
    from ..models.dependency_metrics import DependencyMetrics
    from ..models.external_reference import ExternalReference
    from ..models.license_ import License
    from ..models.organizational_contact import OrganizationalContact
    from ..models.organizational_entity import OrganizationalEntity
    from ..models.project import Project
    from ..models.repository_meta_component import RepositoryMetaComponent
    from ..models.vulnerability import Vulnerability


T = TypeVar("T", bound="Component")


@_attrs_define
class Component:
    """
    Attributes:
        project (Project):
        uuid (UUID):
        authors (Union[Unset, list['OrganizationalContact']]):
        publisher (Union[Unset, str]):
        supplier (Union[Unset, OrganizationalEntity]):
        group (Union[Unset, str]):
        name (Union[Unset, str]):
        version (Union[Unset, str]):
        classifier (Union[Unset, ComponentClassifier]):
        filename (Union[Unset, str]):
        extension (Union[Unset, str]):
        md5 (Union[Unset, str]):
        sha1 (Union[Unset, str]):
        sha256 (Union[Unset, str]):
        sha384 (Union[Unset, str]):
        sha512 (Union[Unset, str]):
        sha3_256 (Union[Unset, str]):
        sha3_384 (Union[Unset, str]):
        sha3_512 (Union[Unset, str]):
        blake2b_256 (Union[Unset, str]):
        blake2b_384 (Union[Unset, str]):
        blake2b_512 (Union[Unset, str]):
        blake3 (Union[Unset, str]):
        cpe (Union[Unset, str]):
        purl (Union[Unset, str]):
        purl_coordinates (Union[Unset, str]):
        swid_tag_id (Union[Unset, str]):
        description (Union[Unset, str]):
        copyright_ (Union[Unset, str]):
        license_ (Union[Unset, str]):
        license_expression (Union[Unset, str]):
        license_url (Union[Unset, str]):
        resolved_license (Union[Unset, License]):
        direct_dependencies (Union[Unset, str]):
        external_references (Union[Unset, list['ExternalReference']]):
        parent (Union[Unset, Component]):
        children (Union[Unset, list['Component']]):
        properties (Union[Unset, list['ComponentProperty']]):
        vulnerabilities (Union[Unset, list['Vulnerability']]):
        last_inherited_risk_score (Union[Unset, float]):
        notes (Union[Unset, str]):
        author (Union[Unset, str]):
        metrics (Union[Unset, DependencyMetrics]):
        repository_meta (Union[Unset, RepositoryMetaComponent]):
        dependency_graph (Union[Unset, list[str]]):
        expand_dependency_graph (Union[Unset, bool]):
        is_internal (Union[Unset, bool]):
    """

    project: "Project"
    uuid: UUID
    authors: Union[Unset, list["OrganizationalContact"]] = UNSET
    publisher: Union[Unset, str] = UNSET
    supplier: Union[Unset, "OrganizationalEntity"] = UNSET
    group: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    classifier: Union[Unset, ComponentClassifier] = UNSET
    filename: Union[Unset, str] = UNSET
    extension: Union[Unset, str] = UNSET
    md5: Union[Unset, str] = UNSET
    sha1: Union[Unset, str] = UNSET
    sha256: Union[Unset, str] = UNSET
    sha384: Union[Unset, str] = UNSET
    sha512: Union[Unset, str] = UNSET
    sha3_256: Union[Unset, str] = UNSET
    sha3_384: Union[Unset, str] = UNSET
    sha3_512: Union[Unset, str] = UNSET
    blake2b_256: Union[Unset, str] = UNSET
    blake2b_384: Union[Unset, str] = UNSET
    blake2b_512: Union[Unset, str] = UNSET
    blake3: Union[Unset, str] = UNSET
    cpe: Union[Unset, str] = UNSET
    purl: Union[Unset, str] = UNSET
    purl_coordinates: Union[Unset, str] = UNSET
    swid_tag_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    copyright_: Union[Unset, str] = UNSET
    license_: Union[Unset, str] = UNSET
    license_expression: Union[Unset, str] = UNSET
    license_url: Union[Unset, str] = UNSET
    resolved_license: Union[Unset, "License"] = UNSET
    direct_dependencies: Union[Unset, str] = UNSET
    external_references: Union[Unset, list["ExternalReference"]] = UNSET
    parent: Union[Unset, "Component"] = UNSET
    children: Union[Unset, list["Component"]] = UNSET
    properties: Union[Unset, list["ComponentProperty"]] = UNSET
    vulnerabilities: Union[Unset, list["Vulnerability"]] = UNSET
    last_inherited_risk_score: Union[Unset, float] = UNSET
    notes: Union[Unset, str] = UNSET
    author: Union[Unset, str] = UNSET
    metrics: Union[Unset, "DependencyMetrics"] = UNSET
    repository_meta: Union[Unset, "RepositoryMetaComponent"] = UNSET
    dependency_graph: Union[Unset, list[str]] = UNSET
    expand_dependency_graph: Union[Unset, bool] = UNSET
    is_internal: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project.to_dict()

        uuid = str(self.uuid)

        authors: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.authors, Unset):
            authors = []
            for authors_item_data in self.authors:
                authors_item = authors_item_data.to_dict()
                authors.append(authors_item)

        publisher = self.publisher

        supplier: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.supplier, Unset):
            supplier = self.supplier.to_dict()

        group = self.group

        name = self.name

        version = self.version

        classifier: Union[Unset, str] = UNSET
        if not isinstance(self.classifier, Unset):
            classifier = self.classifier.value

        filename = self.filename

        extension = self.extension

        md5 = self.md5

        sha1 = self.sha1

        sha256 = self.sha256

        sha384 = self.sha384

        sha512 = self.sha512

        sha3_256 = self.sha3_256

        sha3_384 = self.sha3_384

        sha3_512 = self.sha3_512

        blake2b_256 = self.blake2b_256

        blake2b_384 = self.blake2b_384

        blake2b_512 = self.blake2b_512

        blake3 = self.blake3

        cpe = self.cpe

        purl = self.purl

        purl_coordinates = self.purl_coordinates

        swid_tag_id = self.swid_tag_id

        description = self.description

        copyright_ = self.copyright_

        license_ = self.license_

        license_expression = self.license_expression

        license_url = self.license_url

        resolved_license: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.resolved_license, Unset):
            resolved_license = self.resolved_license.to_dict()

        direct_dependencies = self.direct_dependencies

        external_references: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.external_references, Unset):
            external_references = []
            for external_references_item_data in self.external_references:
                external_references_item = external_references_item_data.to_dict()
                external_references.append(external_references_item)

        parent: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        children: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.children, Unset):
            children = []
            for children_item_data in self.children:
                children_item = children_item_data.to_dict()
                children.append(children_item)

        properties: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        vulnerabilities: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.vulnerabilities, Unset):
            vulnerabilities = []
            for vulnerabilities_item_data in self.vulnerabilities:
                vulnerabilities_item = vulnerabilities_item_data.to_dict()
                vulnerabilities.append(vulnerabilities_item)

        last_inherited_risk_score = self.last_inherited_risk_score

        notes = self.notes

        author = self.author

        metrics: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metrics, Unset):
            metrics = self.metrics.to_dict()

        repository_meta: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.repository_meta, Unset):
            repository_meta = self.repository_meta.to_dict()

        dependency_graph: Union[Unset, list[str]] = UNSET
        if not isinstance(self.dependency_graph, Unset):
            dependency_graph = self.dependency_graph

        expand_dependency_graph = self.expand_dependency_graph

        is_internal = self.is_internal

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project": project,
                "uuid": uuid,
            }
        )
        if authors is not UNSET:
            field_dict["authors"] = authors
        if publisher is not UNSET:
            field_dict["publisher"] = publisher
        if supplier is not UNSET:
            field_dict["supplier"] = supplier
        if group is not UNSET:
            field_dict["group"] = group
        if name is not UNSET:
            field_dict["name"] = name
        if version is not UNSET:
            field_dict["version"] = version
        if classifier is not UNSET:
            field_dict["classifier"] = classifier
        if filename is not UNSET:
            field_dict["filename"] = filename
        if extension is not UNSET:
            field_dict["extension"] = extension
        if md5 is not UNSET:
            field_dict["md5"] = md5
        if sha1 is not UNSET:
            field_dict["sha1"] = sha1
        if sha256 is not UNSET:
            field_dict["sha256"] = sha256
        if sha384 is not UNSET:
            field_dict["sha384"] = sha384
        if sha512 is not UNSET:
            field_dict["sha512"] = sha512
        if sha3_256 is not UNSET:
            field_dict["sha3_256"] = sha3_256
        if sha3_384 is not UNSET:
            field_dict["sha3_384"] = sha3_384
        if sha3_512 is not UNSET:
            field_dict["sha3_512"] = sha3_512
        if blake2b_256 is not UNSET:
            field_dict["blake2b_256"] = blake2b_256
        if blake2b_384 is not UNSET:
            field_dict["blake2b_384"] = blake2b_384
        if blake2b_512 is not UNSET:
            field_dict["blake2b_512"] = blake2b_512
        if blake3 is not UNSET:
            field_dict["blake3"] = blake3
        if cpe is not UNSET:
            field_dict["cpe"] = cpe
        if purl is not UNSET:
            field_dict["purl"] = purl
        if purl_coordinates is not UNSET:
            field_dict["purlCoordinates"] = purl_coordinates
        if swid_tag_id is not UNSET:
            field_dict["swidTagId"] = swid_tag_id
        if description is not UNSET:
            field_dict["description"] = description
        if copyright_ is not UNSET:
            field_dict["copyright"] = copyright_
        if license_ is not UNSET:
            field_dict["license"] = license_
        if license_expression is not UNSET:
            field_dict["licenseExpression"] = license_expression
        if license_url is not UNSET:
            field_dict["licenseUrl"] = license_url
        if resolved_license is not UNSET:
            field_dict["resolvedLicense"] = resolved_license
        if direct_dependencies is not UNSET:
            field_dict["directDependencies"] = direct_dependencies
        if external_references is not UNSET:
            field_dict["externalReferences"] = external_references
        if parent is not UNSET:
            field_dict["parent"] = parent
        if children is not UNSET:
            field_dict["children"] = children
        if properties is not UNSET:
            field_dict["properties"] = properties
        if vulnerabilities is not UNSET:
            field_dict["vulnerabilities"] = vulnerabilities
        if last_inherited_risk_score is not UNSET:
            field_dict["lastInheritedRiskScore"] = last_inherited_risk_score
        if notes is not UNSET:
            field_dict["notes"] = notes
        if author is not UNSET:
            field_dict["author"] = author
        if metrics is not UNSET:
            field_dict["metrics"] = metrics
        if repository_meta is not UNSET:
            field_dict["repositoryMeta"] = repository_meta
        if dependency_graph is not UNSET:
            field_dict["dependencyGraph"] = dependency_graph
        if expand_dependency_graph is not UNSET:
            field_dict["expandDependencyGraph"] = expand_dependency_graph
        if is_internal is not UNSET:
            field_dict["isInternal"] = is_internal

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.component_property import ComponentProperty
        from ..models.dependency_metrics import DependencyMetrics
        from ..models.external_reference import ExternalReference
        from ..models.license_ import License
        from ..models.organizational_contact import OrganizationalContact
        from ..models.organizational_entity import OrganizationalEntity
        from ..models.project import Project
        from ..models.repository_meta_component import RepositoryMetaComponent
        from ..models.vulnerability import Vulnerability

        d = dict(src_dict)
        project = Project.from_dict(d.pop("project"))

        uuid = UUID(d.pop("uuid"))

        authors = []
        _authors = d.pop("authors", UNSET)
        for authors_item_data in _authors or []:
            authors_item = OrganizationalContact.from_dict(authors_item_data)

            authors.append(authors_item)

        publisher = d.pop("publisher", UNSET)

        _supplier = d.pop("supplier", UNSET)
        supplier: Union[Unset, OrganizationalEntity]
        if isinstance(_supplier, Unset):
            supplier = UNSET
        else:
            supplier = OrganizationalEntity.from_dict(_supplier)

        group = d.pop("group", UNSET)

        name = d.pop("name", UNSET)

        version = d.pop("version", UNSET)

        _classifier = d.pop("classifier", UNSET)
        classifier: Union[Unset, ComponentClassifier]
        if isinstance(_classifier, Unset):
            classifier = UNSET
        else:
            classifier = ComponentClassifier(_classifier)

        filename = d.pop("filename", UNSET)

        extension = d.pop("extension", UNSET)

        md5 = d.pop("md5", UNSET)

        sha1 = d.pop("sha1", UNSET)

        sha256 = d.pop("sha256", UNSET)

        sha384 = d.pop("sha384", UNSET)

        sha512 = d.pop("sha512", UNSET)

        sha3_256 = d.pop("sha3_256", UNSET)

        sha3_384 = d.pop("sha3_384", UNSET)

        sha3_512 = d.pop("sha3_512", UNSET)

        blake2b_256 = d.pop("blake2b_256", UNSET)

        blake2b_384 = d.pop("blake2b_384", UNSET)

        blake2b_512 = d.pop("blake2b_512", UNSET)

        blake3 = d.pop("blake3", UNSET)

        cpe = d.pop("cpe", UNSET)

        purl = d.pop("purl", UNSET)

        purl_coordinates = d.pop("purlCoordinates", UNSET)

        swid_tag_id = d.pop("swidTagId", UNSET)

        description = d.pop("description", UNSET)

        copyright_ = d.pop("copyright", UNSET)

        license_ = d.pop("license", UNSET)

        license_expression = d.pop("licenseExpression", UNSET)

        license_url = d.pop("licenseUrl", UNSET)

        _resolved_license = d.pop("resolvedLicense", UNSET)
        resolved_license: Union[Unset, License]
        if isinstance(_resolved_license, Unset):
            resolved_license = UNSET
        else:
            resolved_license = License.from_dict(_resolved_license)

        direct_dependencies = d.pop("directDependencies", UNSET)

        external_references = []
        _external_references = d.pop("externalReferences", UNSET)
        for external_references_item_data in _external_references or []:
            external_references_item = ExternalReference.from_dict(
                external_references_item_data
            )

            external_references.append(external_references_item)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, Component]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = Component.from_dict(_parent)

        children = []
        _children = d.pop("children", UNSET)
        for children_item_data in _children or []:
            children_item = Component.from_dict(children_item_data)

            children.append(children_item)

        properties = []
        _properties = d.pop("properties", UNSET)
        for properties_item_data in _properties or []:
            properties_item = ComponentProperty.from_dict(properties_item_data)

            properties.append(properties_item)

        vulnerabilities = []
        _vulnerabilities = d.pop("vulnerabilities", UNSET)
        for vulnerabilities_item_data in _vulnerabilities or []:
            vulnerabilities_item = Vulnerability.from_dict(vulnerabilities_item_data)

            vulnerabilities.append(vulnerabilities_item)

        last_inherited_risk_score = d.pop("lastInheritedRiskScore", UNSET)

        notes = d.pop("notes", UNSET)

        author = d.pop("author", UNSET)

        _metrics = d.pop("metrics", UNSET)
        metrics: Union[Unset, DependencyMetrics]
        if isinstance(_metrics, Unset):
            metrics = UNSET
        else:
            metrics = DependencyMetrics.from_dict(_metrics)

        _repository_meta = d.pop("repositoryMeta", UNSET)
        repository_meta: Union[Unset, RepositoryMetaComponent]
        if isinstance(_repository_meta, Unset):
            repository_meta = UNSET
        else:
            repository_meta = RepositoryMetaComponent.from_dict(_repository_meta)

        dependency_graph = cast(list[str], d.pop("dependencyGraph", UNSET))

        expand_dependency_graph = d.pop("expandDependencyGraph", UNSET)

        is_internal = d.pop("isInternal", UNSET)

        component = cls(
            project=project,
            uuid=uuid,
            authors=authors,
            publisher=publisher,
            supplier=supplier,
            group=group,
            name=name,
            version=version,
            classifier=classifier,
            filename=filename,
            extension=extension,
            md5=md5,
            sha1=sha1,
            sha256=sha256,
            sha384=sha384,
            sha512=sha512,
            sha3_256=sha3_256,
            sha3_384=sha3_384,
            sha3_512=sha3_512,
            blake2b_256=blake2b_256,
            blake2b_384=blake2b_384,
            blake2b_512=blake2b_512,
            blake3=blake3,
            cpe=cpe,
            purl=purl,
            purl_coordinates=purl_coordinates,
            swid_tag_id=swid_tag_id,
            description=description,
            copyright_=copyright_,
            license_=license_,
            license_expression=license_expression,
            license_url=license_url,
            resolved_license=resolved_license,
            direct_dependencies=direct_dependencies,
            external_references=external_references,
            parent=parent,
            children=children,
            properties=properties,
            vulnerabilities=vulnerabilities,
            last_inherited_risk_score=last_inherited_risk_score,
            notes=notes,
            author=author,
            metrics=metrics,
            repository_meta=repository_meta,
            dependency_graph=dependency_graph,
            expand_dependency_graph=expand_dependency_graph,
            is_internal=is_internal,
        )

        component.additional_properties = d
        return component

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.finding_attrib_analyzer_identity import FindingAttribAnalyzerIdentity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.component import Component
    from ..models.vulnerability import Vulnerability


T = TypeVar("T", bound="FindingAttrib")


@_attrs_define
class FindingAttrib:
    """
    Attributes:
        attributed_on (Union[Unset, int]): UNIX epoch timestamp in milliseconds
        analyzer_identity (Union[Unset, FindingAttribAnalyzerIdentity]):
        component (Union[Unset, Component]):
        vulnerability (Union[Unset, Vulnerability]):
        alternate_identifier (Union[Unset, str]):
        reference_url (Union[Unset, str]):
        uuid (Union[Unset, UUID]):
    """

    attributed_on: Union[Unset, int] = UNSET
    analyzer_identity: Union[Unset, FindingAttribAnalyzerIdentity] = UNSET
    component: Union[Unset, "Component"] = UNSET
    vulnerability: Union[Unset, "Vulnerability"] = UNSET
    alternate_identifier: Union[Unset, str] = UNSET
    reference_url: Union[Unset, str] = UNSET
    uuid: Union[Unset, UUID] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        attributed_on = self.attributed_on

        analyzer_identity: Union[Unset, str] = UNSET
        if not isinstance(self.analyzer_identity, Unset):
            analyzer_identity = self.analyzer_identity.value

        component: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.component, Unset):
            component = self.component.to_dict()

        vulnerability: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.vulnerability, Unset):
            vulnerability = self.vulnerability.to_dict()

        alternate_identifier = self.alternate_identifier

        reference_url = self.reference_url

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attributed_on is not UNSET:
            field_dict["attributedOn"] = attributed_on
        if analyzer_identity is not UNSET:
            field_dict["analyzerIdentity"] = analyzer_identity
        if component is not UNSET:
            field_dict["component"] = component
        if vulnerability is not UNSET:
            field_dict["vulnerability"] = vulnerability
        if alternate_identifier is not UNSET:
            field_dict["alternateIdentifier"] = alternate_identifier
        if reference_url is not UNSET:
            field_dict["referenceUrl"] = reference_url
        if uuid is not UNSET:
            field_dict["uuid"] = uuid

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.component import Component
        from ..models.vulnerability import Vulnerability

        d = dict(src_dict)
        attributed_on = d.pop("attributedOn", UNSET)

        _analyzer_identity = d.pop("analyzerIdentity", UNSET)
        analyzer_identity: Union[Unset, FindingAttribAnalyzerIdentity]
        if isinstance(_analyzer_identity, Unset):
            analyzer_identity = UNSET
        else:
            analyzer_identity = FindingAttribAnalyzerIdentity(_analyzer_identity)

        _component = d.pop("component", UNSET)
        component: Union[Unset, Component]
        if isinstance(_component, Unset):
            component = UNSET
        else:
            component = Component.from_dict(_component)

        _vulnerability = d.pop("vulnerability", UNSET)
        vulnerability: Union[Unset, Vulnerability]
        if isinstance(_vulnerability, Unset):
            vulnerability = UNSET
        else:
            vulnerability = Vulnerability.from_dict(_vulnerability)

        alternate_identifier = d.pop("alternateIdentifier", UNSET)

        reference_url = d.pop("referenceUrl", UNSET)

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        finding_attrib = cls(
            attributed_on=attributed_on,
            analyzer_identity=analyzer_identity,
            component=component,
            vulnerability=vulnerability,
            alternate_identifier=alternate_identifier,
            reference_url=reference_url,
            uuid=uuid,
        )

        finding_attrib.additional_properties = d
        return finding_attrib

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class NotificationRuleNotifyOnItem(str, Enum):
    ANALYZER = "ANALYZER"
    BOM_CONSUMED = "BOM_CONSUMED"
    BOM_PROCESSED = "BOM_PROCESSED"
    BOM_PROCESSING_FAILED = "BOM_PROCESSING_FAILED"
    BOM_VALIDATION_FAILED = "BOM_VALIDATION_FAILED"
    CONFIGURATION = "CONFIGURATION"
    DATASOURCE_MIRRORING = "DATASOURCE_MIRRORING"
    FILE_SYSTEM = "FILE_SYSTEM"
    INDEXING_SERVICE = "INDEXING_SERVICE"
    INTEGRATION = "INTEGRATION"
    NEW_POLICY_VIOLATIONS_SUMMARY = "NEW_POLICY_VIOLATIONS_SUMMARY"
    NEW_VULNERABILITIES_SUMMARY = "NEW_VULNERABILITIES_SUMMARY"
    NEW_VULNERABILITY = "NEW_VULNERABILITY"
    NEW_VULNERABLE_DEPENDENCY = "NEW_VULNERABLE_DEPENDENCY"
    POLICY_VIOLATION = "POLICY_VIOLATION"
    PROJECT_AUDIT_CHANGE = "PROJECT_AUDIT_CHANGE"
    PROJECT_CREATED = "PROJECT_CREATED"
    REPOSITORY = "REPOSITORY"
    USER_CREATED = "USER_CREATED"
    USER_DELETED = "USER_DELETED"
    VEX_CONSUMED = "VEX_CONSUMED"
    VEX_PROCESSED = "VEX_PROCESSED"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_classification import DataClassification
    from ..models.external_reference import ExternalReference
    from ..models.organizational_entity import OrganizationalEntity
    from ..models.project import Project
    from ..models.vulnerability import Vulnerability


T = TypeVar("T", bound="ServiceComponent")


@_attrs_define
class ServiceComponent:
    """
    Attributes:
        name (str):
        project (Project):
        uuid (UUID):
        provider (Union[Unset, OrganizationalEntity]):
        group (Union[Unset, str]):
        version (Union[Unset, str]):
        description (Union[Unset, str]):
        endpoints (Union[Unset, list[str]]):
        authenticated (Union[Unset, bool]):
        crosses_trust_boundary (Union[Unset, bool]):
        data (Union[Unset, list['DataClassification']]):
        external_references (Union[Unset, list['ExternalReference']]):
        parent (Union[Unset, ServiceComponent]):
        children (Union[Unset, list['ServiceComponent']]):
        vulnerabilities (Union[Unset, list['Vulnerability']]):
        last_inherited_risk_score (Union[Unset, float]):
        notes (Union[Unset, str]):
        bom_ref (Union[Unset, str]):
    """

    name: str
    project: "Project"
    uuid: UUID
    provider: Union[Unset, "OrganizationalEntity"] = UNSET
    group: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    endpoints: Union[Unset, list[str]] = UNSET
    authenticated: Union[Unset, bool] = UNSET
    crosses_trust_boundary: Union[Unset, bool] = UNSET
    data: Union[Unset, list["DataClassification"]] = UNSET
    external_references: Union[Unset, list["ExternalReference"]] = UNSET
    parent: Union[Unset, "ServiceComponent"] = UNSET
    children: Union[Unset, list["ServiceComponent"]] = UNSET
    vulnerabilities: Union[Unset, list["Vulnerability"]] = UNSET
    last_inherited_risk_score: Union[Unset, float] = UNSET
    notes: Union[Unset, str] = UNSET
    bom_ref: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        project = self.project.to_dict()

        uuid = str(self.uuid)

        provider: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.provider, Unset):
            provider = self.provider.to_dict()

        group = self.group

        version = self.version

        description = self.description

        endpoints: Union[Unset, list[str]] = UNSET
        if not isinstance(self.endpoints, Unset):
            endpoints = self.endpoints

        authenticated = self.authenticated

        crosses_trust_boundary = self.crosses_trust_boundary

        data: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.data, Unset):
            data = []
            for data_item_data in self.data:
                data_item = data_item_data.to_dict()
                data.append(data_item)

        external_references: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.external_references, Unset):
            external_references = []
            for external_references_item_data in self.external_references:
                external_references_item = external_references_item_data.to_dict()
                external_references.append(external_references_item)

        parent: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        children: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.children, Unset):
            children = []
            for children_item_data in self.children:
                children_item = children_item_data.to_dict()
                children.append(children_item)

        vulnerabilities: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.vulnerabilities, Unset):
            vulnerabilities = []
            for vulnerabilities_item_data in self.vulnerabilities:
                vulnerabilities_item = vulnerabilities_item_data.to_dict()
                vulnerabilities.append(vulnerabilities_item)

        last_inherited_risk_score = self.last_inherited_risk_score

        notes = self.notes

        bom_ref = self.bom_ref

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "project": project,
                "uuid": uuid,
            }
        )
        if provider is not UNSET:
            field_dict["provider"] = provider
        if group is not UNSET:
            field_dict["group"] = group
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if endpoints is not UNSET:
            field_dict["endpoints"] = endpoints
        if authenticated is not UNSET:
            field_dict["authenticated"] = authenticated
        if crosses_trust_boundary is not UNSET:
            field_dict["crossesTrustBoundary"] = crosses_trust_boundary
        if data is not UNSET:
            field_dict["data"] = data
        if external_references is not UNSET:
            field_dict["externalReferences"] = external_references
        if parent is not UNSET:
            field_dict["parent"] = parent
        if children is not UNSET:
            field_dict["children"] = children
        if vulnerabilities is not UNSET:
            field_dict["vulnerabilities"] = vulnerabilities
        if last_inherited_risk_score is not UNSET:
            field_dict["lastInheritedRiskScore"] = last_inherited_risk_score
        if notes is not UNSET:
            field_dict["notes"] = notes
        if bom_ref is not UNSET:
            field_dict["bomRef"] = bom_ref

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.data_classification import DataClassification
        from ..models.external_reference import ExternalReference
        from ..models.organizational_entity import OrganizationalEntity
        from ..models.project import Project
        from ..models.vulnerability import Vulnerability

        d = dict(src_dict)
        name = d.pop("name")

        project = Project.from_dict(d.pop("project"))

        uuid = UUID(d.pop("uuid"))

        _provider = d.pop("provider", UNSET)
        provider: Union[Unset, OrganizationalEntity]
        if isinstance(_provider, Unset):
            provider = UNSET
        else:
            provider = OrganizationalEntity.from_dict(_provider)

        group = d.pop("group", UNSET)

        version = d.pop("version", UNSET)

        description = d.pop("description", UNSET)

        endpoints = cast(list[str], d.pop("endpoints", UNSET))

        authenticated = d.pop("authenticated", UNSET)

        crosses_trust_boundary = d.pop("crossesTrustBoundary", UNSET)

        data = []
        _data = d.pop("data", UNSET)
        for data_item_data in _data or []:
            data_item = DataClassification.from_dict(data_item_data)

            data.append(data_item)

        external_references = []
        _external_references = d.pop("externalReferences", UNSET)
        for external_references_item_data in _external_references or []:
            external_references_item = ExternalReference.from_dict(
                external_references_item_data
            )

            external_references.append(external_references_item)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, ServiceComponent]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = ServiceComponent.from_dict(_parent)

        children = []
        _children = d.pop("children", UNSET)
        for children_item_data in _children or []:
            children_item = ServiceComponent.from_dict(children_item_data)

            children.append(children_item)

        vulnerabilities = []
        _vulnerabilities = d.pop("vulnerabilities", UNSET)
        for vulnerabilities_item_data in _vulnerabilities or []:
            vulnerabilities_item = Vulnerability.from_dict(vulnerabilities_item_data)

            vulnerabilities.append(vulnerabilities_item)

        last_inherited_risk_score = d.pop("lastInheritedRiskScore", UNSET)

        notes = d.pop("notes", UNSET)

        bom_ref = d.pop("bomRef", UNSET)

        service_component = cls(
            name=name,
            project=project,
            uuid=uuid,
            provider=provider,
            group=group,
            version=version,
            description=description,
            endpoints=endpoints,
            authenticated=authenticated,
            crosses_trust_boundary=crosses_trust_boundary,
            data=data,
            external_references=external_references,
            parent=parent,
            children=children,
            vulnerabilities=vulnerabilities,
            last_inherited_risk_score=last_inherited_risk_score,
            notes=notes,
            bom_ref=bom_ref,
        )

        service_component.additional_properties = d
        return service_component

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.analysis_request_analysis_justification import (
    AnalysisRequestAnalysisJustification,
)
from ..models.analysis_request_analysis_response import AnalysisRequestAnalysisResponse
from ..models.analysis_request_analysis_state import AnalysisRequestAnalysisState
from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalysisRequest")


@_attrs_define
class AnalysisRequest:
    """
    Attributes:
        component (str):
        vulnerability (str):
        project (Union[Unset, str]):
        analysis_state (Union[Unset, AnalysisRequestAnalysisState]):
        analysis_justification (Union[Unset, AnalysisRequestAnalysisJustification]):
        analysis_response (Union[Unset, AnalysisRequestAnalysisResponse]):
        analysis_details (Union[Unset, str]):
        comment (Union[Unset, str]):
        is_suppressed (Union[Unset, bool]):
        suppressed (Union[Unset, bool]):
    """

    component: str
    vulnerability: str
    project: Union[Unset, str] = UNSET
    analysis_state: Union[Unset, AnalysisRequestAnalysisState] = UNSET
    analysis_justification: Union[Unset, AnalysisRequestAnalysisJustification] = UNSET
    analysis_response: Union[Unset, AnalysisRequestAnalysisResponse] = UNSET
    analysis_details: Union[Unset, str] = UNSET
    comment: Union[Unset, str] = UNSET
    is_suppressed: Union[Unset, bool] = UNSET
    suppressed: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        component = self.component

        vulnerability = self.vulnerability

        project = self.project

        analysis_state: Union[Unset, str] = UNSET
        if not isinstance(self.analysis_state, Unset):
            analysis_state = self.analysis_state.value

        analysis_justification: Union[Unset, str] = UNSET
        if not isinstance(self.analysis_justification, Unset):
            analysis_justification = self.analysis_justification.value

        analysis_response: Union[Unset, str] = UNSET
        if not isinstance(self.analysis_response, Unset):
            analysis_response = self.analysis_response.value

        analysis_details = self.analysis_details

        comment = self.comment

        is_suppressed = self.is_suppressed

        suppressed = self.suppressed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "component": component,
                "vulnerability": vulnerability,
            }
        )
        if project is not UNSET:
            field_dict["project"] = project
        if analysis_state is not UNSET:
            field_dict["analysisState"] = analysis_state
        if analysis_justification is not UNSET:
            field_dict["analysisJustification"] = analysis_justification
        if analysis_response is not UNSET:
            field_dict["analysisResponse"] = analysis_response
        if analysis_details is not UNSET:
            field_dict["analysisDetails"] = analysis_details
        if comment is not UNSET:
            field_dict["comment"] = comment
        if is_suppressed is not UNSET:
            field_dict["isSuppressed"] = is_suppressed
        if suppressed is not UNSET:
            field_dict["suppressed"] = suppressed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        component = d.pop("component")

        vulnerability = d.pop("vulnerability")

        project = d.pop("project", UNSET)

        _analysis_state = d.pop("analysisState", UNSET)
        analysis_state: Union[Unset, AnalysisRequestAnalysisState]
        if isinstance(_analysis_state, Unset):
            analysis_state = UNSET
        else:
            analysis_state = AnalysisRequestAnalysisState(_analysis_state)

        _analysis_justification = d.pop("analysisJustification", UNSET)
        analysis_justification: Union[Unset, AnalysisRequestAnalysisJustification]
        if isinstance(_analysis_justification, Unset):
            analysis_justification = UNSET
        else:
            analysis_justification = AnalysisRequestAnalysisJustification(
                _analysis_justification
            )

        _analysis_response = d.pop("analysisResponse", UNSET)
        analysis_response: Union[Unset, AnalysisRequestAnalysisResponse]
        if isinstance(_analysis_response, Unset):
            analysis_response = UNSET
        else:
            analysis_response = AnalysisRequestAnalysisResponse(_analysis_response)

        analysis_details = d.pop("analysisDetails", UNSET)

        comment = d.pop("comment", UNSET)

        is_suppressed = d.pop("isSuppressed", UNSET)

        suppressed = d.pop("suppressed", UNSET)

        analysis_request = cls(
            component=component,
            vulnerability=vulnerability,
            project=project,
            analysis_state=analysis_state,
            analysis_justification=analysis_justification,
            analysis_response=analysis_response,
            analysis_details=analysis_details,
            comment=comment,
            is_suppressed=is_suppressed,
            suppressed=suppressed,
        )

        analysis_request.additional_properties = d
        return analysis_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tag_operation_problem_details_errors import (
        TagOperationProblemDetailsErrors,
    )


T = TypeVar("T", bound="TagOperationProblemDetails")


@_attrs_define
class TagOperationProblemDetails:
    """
    Attributes:
        status (int): HTTP status code generated by the origin server for this occurrence of the problem Example: 400.
        title (str): Short, human-readable summary of the problem type Example: Example title.
        detail (str): Human-readable explanation specific to this occurrence of the problem Example: Example detail.
        type_ (Union[Unset, str]): A URI reference that identifies the problem type Example:
            https://api.example.org/foo/bar/example-problem.
        instance (Union[Unset, str]): Reference URI that identifies the specific occurrence of the problem Example:
            https://api.example.org/foo/bar/example-instance.
        errors (Union[Unset, TagOperationProblemDetailsErrors]): Errors encountered during the operation, grouped by tag
            name
    """

    status: int
    title: str
    detail: str
    type_: Union[Unset, str] = UNSET
    instance: Union[Unset, str] = UNSET
    errors: Union[Unset, "TagOperationProblemDetailsErrors"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status

        title = self.title

        detail = self.detail

        type_ = self.type_

        instance = self.instance

        errors: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.errors, Unset):
            errors = self.errors.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "title": title,
                "detail": detail,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_
        if instance is not UNSET:
            field_dict["instance"] = instance
        if errors is not UNSET:
            field_dict["errors"] = errors

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tag_operation_problem_details_errors import (
            TagOperationProblemDetailsErrors,
        )

        d = dict(src_dict)
        status = d.pop("status")

        title = d.pop("title")

        detail = d.pop("detail")

        type_ = d.pop("type", UNSET)

        instance = d.pop("instance", UNSET)

        _errors = d.pop("errors", UNSET)
        errors: Union[Unset, TagOperationProblemDetailsErrors]
        if isinstance(_errors, Unset):
            errors = UNSET
        else:
            errors = TagOperationProblemDetailsErrors.from_dict(_errors)

        tag_operation_problem_details = cls(
            status=status,
            title=title,
            detail=detail,
            type_=type_,
            instance=instance,
            errors=errors,
        )

        tag_operation_problem_details.additional_properties = d
        return tag_operation_problem_details

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AnalysisRequestAnalysisState(str, Enum):
    EXPLOITABLE = "EXPLOITABLE"
    FALSE_POSITIVE = "FALSE_POSITIVE"
    IN_TRIAGE = "IN_TRIAGE"
    NOT_AFFECTED = "NOT_AFFECTED"
    NOT_SET = "NOT_SET"
    RESOLVED = "RESOLVED"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ProjectClassifier(str, Enum):
    APPLICATION = "APPLICATION"
    CONTAINER = "CONTAINER"
    DATA = "DATA"
    DEVICE = "DEVICE"
    DEVICE_DRIVER = "DEVICE_DRIVER"
    FILE = "FILE"
    FIRMWARE = "FIRMWARE"
    FRAMEWORK = "FRAMEWORK"
    LIBRARY = "LIBRARY"
    MACHINE_LEARNING_MODEL = "MACHINE_LEARNING_MODEL"
    NONE = "NONE"
    OPERATING_SYSTEM = "OPERATING_SYSTEM"
    PLATFORM = "PLATFORM"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AnalysisRequestAnalysisJustification(str, Enum):
    CODE_NOT_PRESENT = "CODE_NOT_PRESENT"
    CODE_NOT_REACHABLE = "CODE_NOT_REACHABLE"
    NOT_SET = "NOT_SET"
    PROTECTED_AT_PERIMETER = "PROTECTED_AT_PERIMETER"
    PROTECTED_AT_RUNTIME = "PROTECTED_AT_RUNTIME"
    PROTECTED_BY_COMPILER = "PROTECTED_BY_COMPILER"
    PROTECTED_BY_MITIGATING_CONTROL = "PROTECTED_BY_MITIGATING_CONTROL"
    REQUIRES_CONFIGURATION = "REQUIRES_CONFIGURATION"
    REQUIRES_DEPENDENCY = "REQUIRES_DEPENDENCY"
    REQUIRES_ENVIRONMENT = "REQUIRES_ENVIRONMENT"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiKey")


@_attrs_define
class ApiKey:
    """
    Attributes:
        comment (Union[Unset, str]):
        created (Union[Unset, int]): UNIX epoch timestamp in milliseconds
        last_used (Union[Unset, int]): UNIX epoch timestamp in milliseconds
        public_id (Union[Unset, str]):
        key (Union[Unset, str]):
        legacy (Union[Unset, bool]):
        masked_key (Union[Unset, str]):
    """

    comment: Union[Unset, str] = UNSET
    created: Union[Unset, int] = UNSET
    last_used: Union[Unset, int] = UNSET
    public_id: Union[Unset, str] = UNSET
    key: Union[Unset, str] = UNSET
    legacy: Union[Unset, bool] = UNSET
    masked_key: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        comment = self.comment

        created = self.created

        last_used = self.last_used

        public_id = self.public_id

        key = self.key

        legacy = self.legacy

        masked_key = self.masked_key

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if comment is not UNSET:
            field_dict["comment"] = comment
        if created is not UNSET:
            field_dict["created"] = created
        if last_used is not UNSET:
            field_dict["lastUsed"] = last_used
        if public_id is not UNSET:
            field_dict["publicId"] = public_id
        if key is not UNSET:
            field_dict["key"] = key
        if legacy is not UNSET:
            field_dict["legacy"] = legacy
        if masked_key is not UNSET:
            field_dict["maskedKey"] = masked_key

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        comment = d.pop("comment", UNSET)

        created = d.pop("created", UNSET)

        last_used = d.pop("lastUsed", UNSET)

        public_id = d.pop("publicId", UNSET)

        key = d.pop("key", UNSET)

        legacy = d.pop("legacy", UNSET)

        masked_key = d.pop("maskedKey", UNSET)

        api_key = cls(
            comment=comment,
            created=created,
            last_used=last_used,
            public_id=public_id,
            key=key,
            legacy=legacy,
            masked_key=masked_key,
        )

        api_key.additional_properties = d
        return api_key

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="UploadBomBody")


@_attrs_define
class UploadBomBody:
    """
    Attributes:
        project (Union[Unset, str]):
        auto_create (Union[Unset, bool]):  Default: False.
        project_name (Union[Unset, str]):
        project_version (Union[Unset, str]):
        project_tags (Union[Unset, str]):
        parent_name (Union[Unset, str]):
        parent_version (Union[Unset, str]):
        parent_uuid (Union[Unset, str]):
        is_latest (Union[Unset, bool]):  Default: False.
        bom (Union[Unset, str]):
    """

    project: Union[Unset, str] = UNSET
    auto_create: Union[Unset, bool] = False
    project_name: Union[Unset, str] = UNSET
    project_version: Union[Unset, str] = UNSET
    project_tags: Union[Unset, str] = UNSET
    parent_name: Union[Unset, str] = UNSET
    parent_version: Union[Unset, str] = UNSET
    parent_uuid: Union[Unset, str] = UNSET
    is_latest: Union[Unset, bool] = False
    bom: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project

        auto_create = self.auto_create

        project_name = self.project_name

        project_version = self.project_version

        project_tags = self.project_tags

        parent_name = self.parent_name

        parent_version = self.parent_version

        parent_uuid = self.parent_uuid

        is_latest = self.is_latest

        bom = self.bom

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if project is not UNSET:
            field_dict["project"] = project
        if auto_create is not UNSET:
            field_dict["autoCreate"] = auto_create
        if project_name is not UNSET:
            field_dict["projectName"] = project_name
        if project_version is not UNSET:
            field_dict["projectVersion"] = project_version
        if project_tags is not UNSET:
            field_dict["projectTags"] = project_tags
        if parent_name is not UNSET:
            field_dict["parentName"] = parent_name
        if parent_version is not UNSET:
            field_dict["parentVersion"] = parent_version
        if parent_uuid is not UNSET:
            field_dict["parentUUID"] = parent_uuid
        if is_latest is not UNSET:
            field_dict["isLatest"] = is_latest
        if bom is not UNSET:
            field_dict["bom"] = bom

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.project, Unset):
            files.append(("project", (None, str(self.project).encode(), "text/plain")))

        if not isinstance(self.auto_create, Unset):
            files.append(
                ("autoCreate", (None, str(self.auto_create).encode(), "text/plain"))
            )

        if not isinstance(self.project_name, Unset):
            files.append(
                ("projectName", (None, str(self.project_name).encode(), "text/plain"))
            )

        if not isinstance(self.project_version, Unset):
            files.append(
                (
                    "projectVersion",
                    (None, str(self.project_version).encode(), "text/plain"),
                )
            )

        if not isinstance(self.project_tags, Unset):
            files.append(
                ("projectTags", (None, str(self.project_tags).encode(), "text/plain"))
            )

        if not isinstance(self.parent_name, Unset):
            files.append(
                ("parentName", (None, str(self.parent_name).encode(), "text/plain"))
            )

        if not isinstance(self.parent_version, Unset):
            files.append(
                (
                    "parentVersion",
                    (None, str(self.parent_version).encode(), "text/plain"),
                )
            )

        if not isinstance(self.parent_uuid, Unset):
            files.append(
                ("parentUUID", (None, str(self.parent_uuid).encode(), "text/plain"))
            )

        if not isinstance(self.is_latest, Unset):
            files.append(
                ("isLatest", (None, str(self.is_latest).encode(), "text/plain"))
            )

        if not isinstance(self.bom, Unset):
            files.append(("bom", (None, str(self.bom).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        project = d.pop("project", UNSET)

        auto_create = d.pop("autoCreate", UNSET)

        project_name = d.pop("projectName", UNSET)

        project_version = d.pop("projectVersion", UNSET)

        project_tags = d.pop("projectTags", UNSET)

        parent_name = d.pop("parentName", UNSET)

        parent_version = d.pop("parentVersion", UNSET)

        parent_uuid = d.pop("parentUUID", UNSET)

        is_latest = d.pop("isLatest", UNSET)

        bom = d.pop("bom", UNSET)

        upload_bom_body = cls(
            project=project,
            auto_create=auto_create,
            project_name=project_name,
            project_version=project_version,
            project_tags=project_tags,
            parent_name=parent_name,
            parent_version=parent_version,
            parent_uuid=parent_uuid,
            is_latest=is_latest,
            bom=bom,
        )

        upload_bom_body.additional_properties = d
        return upload_bom_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="NotificationPublisher")


@_attrs_define
class NotificationPublisher:
    """
    Attributes:
        name (str):
        publisher_class (str):
        template_mime_type (str):
        uuid (UUID):
        description (Union[Unset, str]):
        template (Union[Unset, str]):
        default_publisher (Union[Unset, bool]):
    """

    name: str
    publisher_class: str
    template_mime_type: str
    uuid: UUID
    description: Union[Unset, str] = UNSET
    template: Union[Unset, str] = UNSET
    default_publisher: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        publisher_class = self.publisher_class

        template_mime_type = self.template_mime_type

        uuid = str(self.uuid)

        description = self.description

        template = self.template

        default_publisher = self.default_publisher

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "publisherClass": publisher_class,
                "templateMimeType": template_mime_type,
                "uuid": uuid,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if template is not UNSET:
            field_dict["template"] = template
        if default_publisher is not UNSET:
            field_dict["defaultPublisher"] = default_publisher

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        publisher_class = d.pop("publisherClass")

        template_mime_type = d.pop("templateMimeType")

        uuid = UUID(d.pop("uuid"))

        description = d.pop("description", UNSET)

        template = d.pop("template", UNSET)

        default_publisher = d.pop("defaultPublisher", UNSET)

        notification_publisher = cls(
            name=name,
            publisher_class=publisher_class,
            template_mime_type=template_mime_type,
            uuid=uuid,
            description=description,
            template=template,
            default_publisher=default_publisher,
        )

        notification_publisher.additional_properties = d
        return notification_publisher

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tag import Tag


T = TypeVar("T", bound="BomSubmitRequest")


@_attrs_define
class BomSubmitRequest:
    """
    Attributes:
        project (str):  Example: 38640b33-4ba9-4733-bdab-cbfc40c6f8aa.
        project_name (str):  Example: Example Application.
        project_version (str):  Example: 1.0.0.
        bom (str): Base64 encoded BOM Example: ewogICJib21Gb3JtYXQiOiAiQ3ljbG9uZURYIiwKICAic3BlY1ZlcnNpb24iOiAiMS40IiwKI
            CAiY29tcG9uZW50cyI6IFsKICAgIHsKICAgICAgInR5cGUiOiAibGlicmFyeSIsCiAgICAgICJuYW1lIjogImFjbWUtbGliIiwKICAgICAgInZlc
            nNpb24iOiAiMS4wLjAiCiAgICB9CiAgXQp9.
        project_tags (Union[Unset, list['Tag']]): Overwrite project tags. Modifying the tags of an existing project
            requires the PORTFOLIO_MANAGEMENT permission.
        auto_create (Union[Unset, bool]):
        parent_uuid (Union[Unset, str]):  Example: 5341f53c-611b-4388-9d9c-731026dc5eec.
        parent_name (Union[Unset, str]):  Example: Example Application Parent.
        parent_version (Union[Unset, str]):  Example: 1.0.0.
        is_latest_project_version (Union[Unset, bool]):
    """

    project: str
    project_name: str
    project_version: str
    bom: str
    project_tags: Union[Unset, list["Tag"]] = UNSET
    auto_create: Union[Unset, bool] = UNSET
    parent_uuid: Union[Unset, str] = UNSET
    parent_name: Union[Unset, str] = UNSET
    parent_version: Union[Unset, str] = UNSET
    is_latest_project_version: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project

        project_name = self.project_name

        project_version = self.project_version

        bom = self.bom

        project_tags: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.project_tags, Unset):
            project_tags = []
            for project_tags_item_data in self.project_tags:
                project_tags_item = project_tags_item_data.to_dict()
                project_tags.append(project_tags_item)

        auto_create = self.auto_create

        parent_uuid = self.parent_uuid

        parent_name = self.parent_name

        parent_version = self.parent_version

        is_latest_project_version = self.is_latest_project_version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project": project,
                "projectName": project_name,
                "projectVersion": project_version,
                "bom": bom,
            }
        )
        if project_tags is not UNSET:
            field_dict["projectTags"] = project_tags
        if auto_create is not UNSET:
            field_dict["autoCreate"] = auto_create
        if parent_uuid is not UNSET:
            field_dict["parentUUID"] = parent_uuid
        if parent_name is not UNSET:
            field_dict["parentName"] = parent_name
        if parent_version is not UNSET:
            field_dict["parentVersion"] = parent_version
        if is_latest_project_version is not UNSET:
            field_dict["isLatestProjectVersion"] = is_latest_project_version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tag import Tag

        d = dict(src_dict)
        project = d.pop("project")

        project_name = d.pop("projectName")

        project_version = d.pop("projectVersion")

        bom = d.pop("bom")

        project_tags = []
        _project_tags = d.pop("projectTags", UNSET)
        for project_tags_item_data in _project_tags or []:
            project_tags_item = Tag.from_dict(project_tags_item_data)

            project_tags.append(project_tags_item)

        auto_create = d.pop("autoCreate", UNSET)

        parent_uuid = d.pop("parentUUID", UNSET)

        parent_name = d.pop("parentName", UNSET)

        parent_version = d.pop("parentVersion", UNSET)

        is_latest_project_version = d.pop("isLatestProjectVersion", UNSET)

        bom_submit_request = cls(
            project=project,
            project_name=project_name,
            project_version=project_version,
            bom=bom,
            project_tags=project_tags,
            auto_create=auto_create,
            parent_uuid=parent_uuid,
            parent_name=parent_name,
            parent_version=parent_version,
            is_latest_project_version=is_latest_project_version,
        )

        bom_submit_request.additional_properties = d
        return bom_submit_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.repository_meta_component_repository_type import (
    RepositoryMetaComponentRepositoryType,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="RepositoryMetaComponent")


@_attrs_define
class RepositoryMetaComponent:
    """
    Attributes:
        repository_type (RepositoryMetaComponentRepositoryType):
        name (str):
        latest_version (str):
        published (int): UNIX epoch timestamp in milliseconds
        last_check (int): UNIX epoch timestamp in milliseconds
        namespace (Union[Unset, str]):
    """

    repository_type: RepositoryMetaComponentRepositoryType
    name: str
    latest_version: str
    published: int
    last_check: int
    namespace: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        repository_type = self.repository_type.value

        name = self.name

        latest_version = self.latest_version

        published = self.published

        last_check = self.last_check

        namespace = self.namespace

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "repositoryType": repository_type,
                "name": name,
                "latestVersion": latest_version,
                "published": published,
                "lastCheck": last_check,
            }
        )
        if namespace is not UNSET:
            field_dict["namespace"] = namespace

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        repository_type = RepositoryMetaComponentRepositoryType(d.pop("repositoryType"))

        name = d.pop("name")

        latest_version = d.pop("latestVersion")

        published = d.pop("published")

        last_check = d.pop("lastCheck")

        namespace = d.pop("namespace", UNSET)

        repository_meta_component = cls(
            repository_type=repository_type,
            name=name,
            latest_version=latest_version,
            published=published,
            last_check=last_check,
            namespace=namespace,
        )

        repository_meta_component.additional_properties = d
        return repository_meta_component

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.policy_violation_type import PolicyViolationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.component import Component
    from ..models.policy_condition import PolicyCondition
    from ..models.project import Project
    from ..models.violation_analysis import ViolationAnalysis


T = TypeVar("T", bound="PolicyViolation")


@_attrs_define
class PolicyViolation:
    """
    Attributes:
        timestamp (int): UNIX epoch timestamp in milliseconds
        uuid (UUID):
        type_ (Union[Unset, PolicyViolationType]):
        project (Union[Unset, Project]):
        component (Union[Unset, Component]):
        policy_condition (Union[Unset, PolicyCondition]):
        text (Union[Unset, str]):
        analysis (Union[Unset, ViolationAnalysis]):
    """

    timestamp: int
    uuid: UUID
    type_: Union[Unset, PolicyViolationType] = UNSET
    project: Union[Unset, "Project"] = UNSET
    component: Union[Unset, "Component"] = UNSET
    policy_condition: Union[Unset, "PolicyCondition"] = UNSET
    text: Union[Unset, str] = UNSET
    analysis: Union[Unset, "ViolationAnalysis"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        timestamp = self.timestamp

        uuid = str(self.uuid)

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        project: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        component: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.component, Unset):
            component = self.component.to_dict()

        policy_condition: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.policy_condition, Unset):
            policy_condition = self.policy_condition.to_dict()

        text = self.text

        analysis: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.analysis, Unset):
            analysis = self.analysis.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "timestamp": timestamp,
                "uuid": uuid,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_
        if project is not UNSET:
            field_dict["project"] = project
        if component is not UNSET:
            field_dict["component"] = component
        if policy_condition is not UNSET:
            field_dict["policyCondition"] = policy_condition
        if text is not UNSET:
            field_dict["text"] = text
        if analysis is not UNSET:
            field_dict["analysis"] = analysis

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.component import Component
        from ..models.policy_condition import PolicyCondition
        from ..models.project import Project
        from ..models.violation_analysis import ViolationAnalysis

        d = dict(src_dict)
        timestamp = d.pop("timestamp")

        uuid = UUID(d.pop("uuid"))

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, PolicyViolationType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = PolicyViolationType(_type_)

        _project = d.pop("project", UNSET)
        project: Union[Unset, Project]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = Project.from_dict(_project)

        _component = d.pop("component", UNSET)
        component: Union[Unset, Component]
        if isinstance(_component, Unset):
            component = UNSET
        else:
            component = Component.from_dict(_component)

        _policy_condition = d.pop("policyCondition", UNSET)
        policy_condition: Union[Unset, PolicyCondition]
        if isinstance(_policy_condition, Unset):
            policy_condition = UNSET
        else:
            policy_condition = PolicyCondition.from_dict(_policy_condition)

        text = d.pop("text", UNSET)

        _analysis = d.pop("analysis", UNSET)
        analysis: Union[Unset, ViolationAnalysis]
        if isinstance(_analysis, Unset):
            analysis = UNSET
        else:
            analysis = ViolationAnalysis.from_dict(_analysis)

        policy_violation = cls(
            timestamp=timestamp,
            uuid=uuid,
            type_=type_,
            project=project,
            component=component,
            policy_condition=policy_condition,
            text=text,
            analysis=analysis,
        )

        policy_violation.additional_properties = d
        return policy_violation

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.cwe import Cwe
    from ..models.vulnerability_alias import VulnerabilityAlias


T = TypeVar("T", bound="FindingVulnerability")


@_attrs_define
class FindingVulnerability:
    """
    Attributes:
        uuid (Union[Unset, str]):
        source (Union[Unset, str]):
        vuln_id (Union[Unset, str]):
        description (Union[Unset, str]):
        cvss_v2_base_score (Union[Unset, float]):
        cvss_v3_base_score (Union[Unset, float]):
        severity (Union[Unset, str]):
        severity_rank (Union[Unset, int]):
        epss_score (Union[Unset, float]):
        epss_percentile (Union[Unset, float]):
        cwe_id (Union[Unset, int]):
        cwe_name (Union[Unset, str]):
        cwes (Union[Unset, list['Cwe']]):
        recommendation (Union[Unset, str]):
        published (Union[Unset, int]):
        aliases (Union[Unset, list['VulnerabilityAlias']]):
    """

    uuid: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    vuln_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    cvss_v2_base_score: Union[Unset, float] = UNSET
    cvss_v3_base_score: Union[Unset, float] = UNSET
    severity: Union[Unset, str] = UNSET
    severity_rank: Union[Unset, int] = UNSET
    epss_score: Union[Unset, float] = UNSET
    epss_percentile: Union[Unset, float] = UNSET
    cwe_id: Union[Unset, int] = UNSET
    cwe_name: Union[Unset, str] = UNSET
    cwes: Union[Unset, list["Cwe"]] = UNSET
    recommendation: Union[Unset, str] = UNSET
    published: Union[Unset, int] = UNSET
    aliases: Union[Unset, list["VulnerabilityAlias"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = self.uuid

        source = self.source

        vuln_id = self.vuln_id

        description = self.description

        cvss_v2_base_score = self.cvss_v2_base_score

        cvss_v3_base_score = self.cvss_v3_base_score

        severity = self.severity

        severity_rank = self.severity_rank

        epss_score = self.epss_score

        epss_percentile = self.epss_percentile

        cwe_id = self.cwe_id

        cwe_name = self.cwe_name

        cwes: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.cwes, Unset):
            cwes = []
            for cwes_item_data in self.cwes:
                cwes_item = cwes_item_data.to_dict()
                cwes.append(cwes_item)

        recommendation = self.recommendation

        published = self.published

        aliases: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.aliases, Unset):
            aliases = []
            for aliases_item_data in self.aliases:
                aliases_item = aliases_item_data.to_dict()
                aliases.append(aliases_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if source is not UNSET:
            field_dict["source"] = source
        if vuln_id is not UNSET:
            field_dict["vulnId"] = vuln_id
        if description is not UNSET:
            field_dict["description"] = description
        if cvss_v2_base_score is not UNSET:
            field_dict["cvssV2BaseScore"] = cvss_v2_base_score
        if cvss_v3_base_score is not UNSET:
            field_dict["cvssV3BaseScore"] = cvss_v3_base_score
        if severity is not UNSET:
            field_dict["severity"] = severity
        if severity_rank is not UNSET:
            field_dict["severityRank"] = severity_rank
        if epss_score is not UNSET:
            field_dict["epssScore"] = epss_score
        if epss_percentile is not UNSET:
            field_dict["epssPercentile"] = epss_percentile
        if cwe_id is not UNSET:
            field_dict["cweId"] = cwe_id
        if cwe_name is not UNSET:
            field_dict["cweName"] = cwe_name
        if cwes is not UNSET:
            field_dict["cwes"] = cwes
        if recommendation is not UNSET:
            field_dict["recommendation"] = recommendation
        if published is not UNSET:
            field_dict["published"] = published
        if aliases is not UNSET:
            field_dict["aliases"] = aliases

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.cwe import Cwe
        from ..models.vulnerability_alias import VulnerabilityAlias

        d = dict(src_dict)
        uuid = d.pop("uuid", UNSET)

        source = d.pop("source", UNSET)

        vuln_id = d.pop("vulnId", UNSET)

        description = d.pop("description", UNSET)

        cvss_v2_base_score = d.pop("cvssV2BaseScore", UNSET)

        cvss_v3_base_score = d.pop("cvssV3BaseScore", UNSET)

        severity = d.pop("severity", UNSET)

        severity_rank = d.pop("severityRank", UNSET)

        epss_score = d.pop("epssScore", UNSET)

        epss_percentile = d.pop("epssPercentile", UNSET)

        cwe_id = d.pop("cweId", UNSET)

        cwe_name = d.pop("cweName", UNSET)

        cwes = []
        _cwes = d.pop("cwes", UNSET)
        for cwes_item_data in _cwes or []:
            cwes_item = Cwe.from_dict(cwes_item_data)

            cwes.append(cwes_item)

        recommendation = d.pop("recommendation", UNSET)

        published = d.pop("published", UNSET)

        aliases = []
        _aliases = d.pop("aliases", UNSET)
        for aliases_item_data in _aliases or []:
            aliases_item = VulnerabilityAlias.from_dict(aliases_item_data)

            aliases.append(aliases_item)

        finding_vulnerability = cls(
            uuid=uuid,
            source=source,
            vuln_id=vuln_id,
            description=description,
            cvss_v2_base_score=cvss_v2_base_score,
            cvss_v3_base_score=cvss_v3_base_score,
            severity=severity,
            severity_rank=severity_rank,
            epss_score=epss_score,
            epss_percentile=epss_percentile,
            cwe_id=cwe_id,
            cwe_name=cwe_name,
            cwes=cwes,
            recommendation=recommendation,
            published=published,
            aliases=aliases,
        )

        finding_vulnerability.additional_properties = d
        return finding_vulnerability

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalysisComment")


@_attrs_define
class AnalysisComment:
    """
    Attributes:
        timestamp (int): UNIX epoch timestamp in milliseconds
        comment (str):
        commenter (Union[Unset, str]):
    """

    timestamp: int
    comment: str
    commenter: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        timestamp = self.timestamp

        comment = self.comment

        commenter = self.commenter

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "timestamp": timestamp,
                "comment": comment,
            }
        )
        if commenter is not UNSET:
            field_dict["commenter"] = commenter

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        timestamp = d.pop("timestamp")

        comment = d.pop("comment")

        commenter = d.pop("commenter", UNSET)

        analysis_comment = cls(
            timestamp=timestamp,
            comment=comment,
            commenter=commenter,
        )

        analysis_comment.additional_properties = d
        return analysis_comment

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.finding import Finding
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    show_inactive: Union[Unset, bool] = UNSET,
    severity: Union[Unset, str] = UNSET,
    publish_date_from: Union[Unset, str] = UNSET,
    publish_date_to: Union[Unset, str] = UNSET,
    text_search_field: Union[Unset, str] = UNSET,
    text_search_input: Union[Unset, str] = UNSET,
    cvssv_2_from: Union[Unset, str] = UNSET,
    cvssv_2_to: Union[Unset, str] = UNSET,
    cvssv_3_from: Union[Unset, str] = UNSET,
    cvssv_3_to: Union[Unset, str] = UNSET,
    occurrences_from: Union[Unset, str] = UNSET,
    occurrences_to: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["showInactive"] = show_inactive

    params["severity"] = severity

    params["publishDateFrom"] = publish_date_from

    params["publishDateTo"] = publish_date_to

    params["textSearchField"] = text_search_field

    params["textSearchInput"] = text_search_input

    params["cvssv2From"] = cvssv_2_from

    params["cvssv2To"] = cvssv_2_to

    params["cvssv3From"] = cvssv_3_from

    params["cvssv3To"] = cvssv_3_to

    params["occurrencesFrom"] = occurrences_from

    params["occurrencesTo"] = occurrences_to

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/finding/grouped",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, list["Finding"]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Finding.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, list["Finding"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    show_inactive: Union[Unset, bool] = UNSET,
    severity: Union[Unset, str] = UNSET,
    publish_date_from: Union[Unset, str] = UNSET,
    publish_date_to: Union[Unset, str] = UNSET,
    text_search_field: Union[Unset, str] = UNSET,
    text_search_input: Union[Unset, str] = UNSET,
    cvssv_2_from: Union[Unset, str] = UNSET,
    cvssv_2_to: Union[Unset, str] = UNSET,
    cvssv_3_from: Union[Unset, str] = UNSET,
    cvssv_3_to: Union[Unset, str] = UNSET,
    occurrences_from: Union[Unset, str] = UNSET,
    occurrences_to: Union[Unset, str] = UNSET,
) -> Response[Union[Any, list["Finding"]]]:
    """Returns a list of all findings grouped by vulnerability

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        show_inactive (Union[Unset, bool]):
        severity (Union[Unset, str]):
        publish_date_from (Union[Unset, str]):
        publish_date_to (Union[Unset, str]):
        text_search_field (Union[Unset, str]):
        text_search_input (Union[Unset, str]):
        cvssv_2_from (Union[Unset, str]):
        cvssv_2_to (Union[Unset, str]):
        cvssv_3_from (Union[Unset, str]):
        cvssv_3_to (Union[Unset, str]):
        occurrences_from (Union[Unset, str]):
        occurrences_to (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Finding']]]
    """

    kwargs = _get_kwargs(
        show_inactive=show_inactive,
        severity=severity,
        publish_date_from=publish_date_from,
        publish_date_to=publish_date_to,
        text_search_field=text_search_field,
        text_search_input=text_search_input,
        cvssv_2_from=cvssv_2_from,
        cvssv_2_to=cvssv_2_to,
        cvssv_3_from=cvssv_3_from,
        cvssv_3_to=cvssv_3_to,
        occurrences_from=occurrences_from,
        occurrences_to=occurrences_to,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    show_inactive: Union[Unset, bool] = UNSET,
    severity: Union[Unset, str] = UNSET,
    publish_date_from: Union[Unset, str] = UNSET,
    publish_date_to: Union[Unset, str] = UNSET,
    text_search_field: Union[Unset, str] = UNSET,
    text_search_input: Union[Unset, str] = UNSET,
    cvssv_2_from: Union[Unset, str] = UNSET,
    cvssv_2_to: Union[Unset, str] = UNSET,
    cvssv_3_from: Union[Unset, str] = UNSET,
    cvssv_3_to: Union[Unset, str] = UNSET,
    occurrences_from: Union[Unset, str] = UNSET,
    occurrences_to: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, list["Finding"]]]:
    """Returns a list of all findings grouped by vulnerability

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        show_inactive (Union[Unset, bool]):
        severity (Union[Unset, str]):
        publish_date_from (Union[Unset, str]):
        publish_date_to (Union[Unset, str]):
        text_search_field (Union[Unset, str]):
        text_search_input (Union[Unset, str]):
        cvssv_2_from (Union[Unset, str]):
        cvssv_2_to (Union[Unset, str]):
        cvssv_3_from (Union[Unset, str]):
        cvssv_3_to (Union[Unset, str]):
        occurrences_from (Union[Unset, str]):
        occurrences_to (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Finding']]
    """

    return sync_detailed(
        client=client,
        show_inactive=show_inactive,
        severity=severity,
        publish_date_from=publish_date_from,
        publish_date_to=publish_date_to,
        text_search_field=text_search_field,
        text_search_input=text_search_input,
        cvssv_2_from=cvssv_2_from,
        cvssv_2_to=cvssv_2_to,
        cvssv_3_from=cvssv_3_from,
        cvssv_3_to=cvssv_3_to,
        occurrences_from=occurrences_from,
        occurrences_to=occurrences_to,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    show_inactive: Union[Unset, bool] = UNSET,
    severity: Union[Unset, str] = UNSET,
    publish_date_from: Union[Unset, str] = UNSET,
    publish_date_to: Union[Unset, str] = UNSET,
    text_search_field: Union[Unset, str] = UNSET,
    text_search_input: Union[Unset, str] = UNSET,
    cvssv_2_from: Union[Unset, str] = UNSET,
    cvssv_2_to: Union[Unset, str] = UNSET,
    cvssv_3_from: Union[Unset, str] = UNSET,
    cvssv_3_to: Union[Unset, str] = UNSET,
    occurrences_from: Union[Unset, str] = UNSET,
    occurrences_to: Union[Unset, str] = UNSET,
) -> Response[Union[Any, list["Finding"]]]:
    """Returns a list of all findings grouped by vulnerability

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        show_inactive (Union[Unset, bool]):
        severity (Union[Unset, str]):
        publish_date_from (Union[Unset, str]):
        publish_date_to (Union[Unset, str]):
        text_search_field (Union[Unset, str]):
        text_search_input (Union[Unset, str]):
        cvssv_2_from (Union[Unset, str]):
        cvssv_2_to (Union[Unset, str]):
        cvssv_3_from (Union[Unset, str]):
        cvssv_3_to (Union[Unset, str]):
        occurrences_from (Union[Unset, str]):
        occurrences_to (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Finding']]]
    """

    kwargs = _get_kwargs(
        show_inactive=show_inactive,
        severity=severity,
        publish_date_from=publish_date_from,
        publish_date_to=publish_date_to,
        text_search_field=text_search_field,
        text_search_input=text_search_input,
        cvssv_2_from=cvssv_2_from,
        cvssv_2_to=cvssv_2_to,
        cvssv_3_from=cvssv_3_from,
        cvssv_3_to=cvssv_3_to,
        occurrences_from=occurrences_from,
        occurrences_to=occurrences_to,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    show_inactive: Union[Unset, bool] = UNSET,
    severity: Union[Unset, str] = UNSET,
    publish_date_from: Union[Unset, str] = UNSET,
    publish_date_to: Union[Unset, str] = UNSET,
    text_search_field: Union[Unset, str] = UNSET,
    text_search_input: Union[Unset, str] = UNSET,
    cvssv_2_from: Union[Unset, str] = UNSET,
    cvssv_2_to: Union[Unset, str] = UNSET,
    cvssv_3_from: Union[Unset, str] = UNSET,
    cvssv_3_to: Union[Unset, str] = UNSET,
    occurrences_from: Union[Unset, str] = UNSET,
    occurrences_to: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, list["Finding"]]]:
    """Returns a list of all findings grouped by vulnerability

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        show_inactive (Union[Unset, bool]):
        severity (Union[Unset, str]):
        publish_date_from (Union[Unset, str]):
        publish_date_to (Union[Unset, str]):
        text_search_field (Union[Unset, str]):
        text_search_input (Union[Unset, str]):
        cvssv_2_from (Union[Unset, str]):
        cvssv_2_to (Union[Unset, str]):
        cvssv_3_from (Union[Unset, str]):
        cvssv_3_to (Union[Unset, str]):
        occurrences_from (Union[Unset, str]):
        occurrences_to (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Finding']]
    """

    return (
        await asyncio_detailed(
            client=client,
            show_inactive=show_inactive,
            severity=severity,
            publish_date_from=publish_date_from,
            publish_date_to=publish_date_to,
            text_search_field=text_search_field,
            text_search_input=text_search_input,
            cvssv_2_from=cvssv_2_from,
            cvssv_2_to=cvssv_2_to,
            cvssv_3_from=cvssv_3_from,
            cvssv_3_to=cvssv_3_to,
            occurrences_from=occurrences_from,
            occurrences_to=occurrences_to,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Optional, Union, cast
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.analysis import Analysis
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    project: Union[Unset, UUID] = UNSET,
    component: UUID,
    vulnerability: UUID,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_project: Union[Unset, str] = UNSET
    if not isinstance(project, Unset):
        json_project = str(project)
    params["project"] = json_project

    json_component = str(component)
    params["component"] = json_component

    json_vulnerability = str(vulnerability)
    params["vulnerability"] = json_vulnerability

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/analysis",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Analysis, Any]]:
    if response.status_code == 200:
        response_200 = Analysis.from_dict(response.json())

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Analysis, Any]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    project: Union[Unset, UUID] = UNSET,
    component: UUID,
    vulnerability: UUID,
) -> Response[Union[Analysis, Any]]:
    """Retrieves an analysis trail

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        project (Union[Unset, UUID]):
        component (UUID):
        vulnerability (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Analysis, Any]]
    """

    kwargs = _get_kwargs(
        project=project,
        component=component,
        vulnerability=vulnerability,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    project: Union[Unset, UUID] = UNSET,
    component: UUID,
    vulnerability: UUID,
) -> Optional[Union[Analysis, Any]]:
    """Retrieves an analysis trail

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        project (Union[Unset, UUID]):
        component (UUID):
        vulnerability (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Analysis, Any]
    """

    return sync_detailed(
        client=client,
        project=project,
        component=component,
        vulnerability=vulnerability,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    project: Union[Unset, UUID] = UNSET,
    component: UUID,
    vulnerability: UUID,
) -> Response[Union[Analysis, Any]]:
    """Retrieves an analysis trail

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        project (Union[Unset, UUID]):
        component (UUID):
        vulnerability (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Analysis, Any]]
    """

    kwargs = _get_kwargs(
        project=project,
        component=component,
        vulnerability=vulnerability,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    project: Union[Unset, UUID] = UNSET,
    component: UUID,
    vulnerability: UUID,
) -> Optional[Union[Analysis, Any]]:
    """Retrieves an analysis trail

     <p>Requires permission <strong>VIEW_VULNERABILITY</strong></p>

    Args:
        project (Union[Unset, UUID]):
        component (UUID):
        vulnerability (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Analysis, Any]
    """

    return (
        await asyncio_detailed(
            client=client,
            project=project,
            component=component,
            vulnerability=vulnerability,
        )
    ).parsed



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.organizational_contact import OrganizationalContact
    from ..models.organizational_entity import OrganizationalEntity


T = TypeVar("T", bound="ProjectMetadata")


@_attrs_define
class ProjectMetadata:
    """
    Attributes:
        supplier (Union[Unset, OrganizationalEntity]):
        authors (Union[Unset, list['OrganizationalContact']]):
    """

    supplier: Union[Unset, "OrganizationalEntity"] = UNSET
    authors: Union[Unset, list["OrganizationalContact"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        supplier: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.supplier, Unset):
            supplier = self.supplier.to_dict()

        authors: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.authors, Unset):
            authors = []
            for authors_item_data in self.authors:
                authors_item = authors_item_data.to_dict()
                authors.append(authors_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if supplier is not UNSET:
            field_dict["supplier"] = supplier
        if authors is not UNSET:
            field_dict["authors"] = authors

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.organizational_contact import OrganizationalContact
        from ..models.organizational_entity import OrganizationalEntity

        d = dict(src_dict)
        _supplier = d.pop("supplier", UNSET)
        supplier: Union[Unset, OrganizationalEntity]
        if isinstance(_supplier, Unset):
            supplier = UNSET
        else:
            supplier = OrganizationalEntity.from_dict(_supplier)

        authors = []
        _authors = d.pop("authors", UNSET)
        for authors_item_data in _authors or []:
            authors_item = OrganizationalContact.from_dict(authors_item_data)

            authors.append(authors_item)

        project_metadata = cls(
            supplier=supplier,
            authors=authors,
        )

        project_metadata.additional_properties = d
        return project_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProjectVersion")


@_attrs_define
class ProjectVersion:
    """
    Attributes:
        uuid (Union[Unset, UUID]):
        version (Union[Unset, str]):
        active (Union[Unset, bool]):
    """

    uuid: Union[Unset, UUID] = UNSET
    version: Union[Unset, str] = UNSET
    active: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        version = self.version

        active = self.active

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if version is not UNSET:
            field_dict["version"] = version
        if active is not UNSET:
            field_dict["active"] = active

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        version = d.pop("version", UNSET)

        active = d.pop("active", UNSET)

        project_version = cls(
            uuid=uuid,
            version=version,
            active=active,
        )

        project_version.additional_properties = d
        return project_version

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DataClassificationDirection(str, Enum):
    BI_DIRECTIONAL = "BI_DIRECTIONAL"
    INBOUND = "INBOUND"
    OUTBOUND = "OUTBOUND"
    UNKNOWN = "UNKNOWN"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AnalysisAnalysisResponse(str, Enum):
    CAN_NOT_FIX = "CAN_NOT_FIX"
    NOT_SET = "NOT_SET"
    ROLLBACK = "ROLLBACK"
    UPDATE = "UPDATE"
    WILL_NOT_FIX = "WILL_NOT_FIX"
    WORKAROUND_AVAILABLE = "WORKAROUND_AVAILABLE"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PortfolioMetrics")


@_attrs_define
class PortfolioMetrics:
    """
    Attributes:
        critical (int):
        high (int):
        medium (int):
        low (int):
        first_occurrence (int): UNIX epoch timestamp in milliseconds
        last_occurrence (int): UNIX epoch timestamp in milliseconds
        unassigned (Union[Unset, int]):
        vulnerabilities (Union[Unset, int]):
        projects (Union[Unset, int]):
        vulnerable_projects (Union[Unset, int]):
        components (Union[Unset, int]):
        vulnerable_components (Union[Unset, int]):
        suppressed (Union[Unset, int]):
        findings_total (Union[Unset, int]):
        findings_audited (Union[Unset, int]):
        findings_unaudited (Union[Unset, int]):
        inherited_risk_score (Union[Unset, float]):
        policy_violations_fail (Union[Unset, int]):
        policy_violations_warn (Union[Unset, int]):
        policy_violations_info (Union[Unset, int]):
        policy_violations_total (Union[Unset, int]):
        policy_violations_audited (Union[Unset, int]):
        policy_violations_unaudited (Union[Unset, int]):
        policy_violations_security_total (Union[Unset, int]):
        policy_violations_security_audited (Union[Unset, int]):
        policy_violations_security_unaudited (Union[Unset, int]):
        policy_violations_license_total (Union[Unset, int]):
        policy_violations_license_audited (Union[Unset, int]):
        policy_violations_license_unaudited (Union[Unset, int]):
        policy_violations_operational_total (Union[Unset, int]):
        policy_violations_operational_audited (Union[Unset, int]):
        policy_violations_operational_unaudited (Union[Unset, int]):
    """

    critical: int
    high: int
    medium: int
    low: int
    first_occurrence: int
    last_occurrence: int
    unassigned: Union[Unset, int] = UNSET
    vulnerabilities: Union[Unset, int] = UNSET
    projects: Union[Unset, int] = UNSET
    vulnerable_projects: Union[Unset, int] = UNSET
    components: Union[Unset, int] = UNSET
    vulnerable_components: Union[Unset, int] = UNSET
    suppressed: Union[Unset, int] = UNSET
    findings_total: Union[Unset, int] = UNSET
    findings_audited: Union[Unset, int] = UNSET
    findings_unaudited: Union[Unset, int] = UNSET
    inherited_risk_score: Union[Unset, float] = UNSET
    policy_violations_fail: Union[Unset, int] = UNSET
    policy_violations_warn: Union[Unset, int] = UNSET
    policy_violations_info: Union[Unset, int] = UNSET
    policy_violations_total: Union[Unset, int] = UNSET
    policy_violations_audited: Union[Unset, int] = UNSET
    policy_violations_unaudited: Union[Unset, int] = UNSET
    policy_violations_security_total: Union[Unset, int] = UNSET
    policy_violations_security_audited: Union[Unset, int] = UNSET
    policy_violations_security_unaudited: Union[Unset, int] = UNSET
    policy_violations_license_total: Union[Unset, int] = UNSET
    policy_violations_license_audited: Union[Unset, int] = UNSET
    policy_violations_license_unaudited: Union[Unset, int] = UNSET
    policy_violations_operational_total: Union[Unset, int] = UNSET
    policy_violations_operational_audited: Union[Unset, int] = UNSET
    policy_violations_operational_unaudited: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        critical = self.critical

        high = self.high

        medium = self.medium

        low = self.low

        first_occurrence = self.first_occurrence

        last_occurrence = self.last_occurrence

        unassigned = self.unassigned

        vulnerabilities = self.vulnerabilities

        projects = self.projects

        vulnerable_projects = self.vulnerable_projects

        components = self.components

        vulnerable_components = self.vulnerable_components

        suppressed = self.suppressed

        findings_total = self.findings_total

        findings_audited = self.findings_audited

        findings_unaudited = self.findings_unaudited

        inherited_risk_score = self.inherited_risk_score

        policy_violations_fail = self.policy_violations_fail

        policy_violations_warn = self.policy_violations_warn

        policy_violations_info = self.policy_violations_info

        policy_violations_total = self.policy_violations_total

        policy_violations_audited = self.policy_violations_audited

        policy_violations_unaudited = self.policy_violations_unaudited

        policy_violations_security_total = self.policy_violations_security_total

        policy_violations_security_audited = self.policy_violations_security_audited

        policy_violations_security_unaudited = self.policy_violations_security_unaudited

        policy_violations_license_total = self.policy_violations_license_total

        policy_violations_license_audited = self.policy_violations_license_audited

        policy_violations_license_unaudited = self.policy_violations_license_unaudited

        policy_violations_operational_total = self.policy_violations_operational_total

        policy_violations_operational_audited = (
            self.policy_violations_operational_audited
        )

        policy_violations_operational_unaudited = (
            self.policy_violations_operational_unaudited
        )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "critical": critical,
                "high": high,
                "medium": medium,
                "low": low,
                "firstOccurrence": first_occurrence,
                "lastOccurrence": last_occurrence,
            }
        )
        if unassigned is not UNSET:
            field_dict["unassigned"] = unassigned
        if vulnerabilities is not UNSET:
            field_dict["vulnerabilities"] = vulnerabilities
        if projects is not UNSET:
            field_dict["projects"] = projects
        if vulnerable_projects is not UNSET:
            field_dict["vulnerableProjects"] = vulnerable_projects
        if components is not UNSET:
            field_dict["components"] = components
        if vulnerable_components is not UNSET:
            field_dict["vulnerableComponents"] = vulnerable_components
        if suppressed is not UNSET:
            field_dict["suppressed"] = suppressed
        if findings_total is not UNSET:
            field_dict["findingsTotal"] = findings_total
        if findings_audited is not UNSET:
            field_dict["findingsAudited"] = findings_audited
        if findings_unaudited is not UNSET:
            field_dict["findingsUnaudited"] = findings_unaudited
        if inherited_risk_score is not UNSET:
            field_dict["inheritedRiskScore"] = inherited_risk_score
        if policy_violations_fail is not UNSET:
            field_dict["policyViolationsFail"] = policy_violations_fail
        if policy_violations_warn is not UNSET:
            field_dict["policyViolationsWarn"] = policy_violations_warn
        if policy_violations_info is not UNSET:
            field_dict["policyViolationsInfo"] = policy_violations_info
        if policy_violations_total is not UNSET:
            field_dict["policyViolationsTotal"] = policy_violations_total
        if policy_violations_audited is not UNSET:
            field_dict["policyViolationsAudited"] = policy_violations_audited
        if policy_violations_unaudited is not UNSET:
            field_dict["policyViolationsUnaudited"] = policy_violations_unaudited
        if policy_violations_security_total is not UNSET:
            field_dict["policyViolationsSecurityTotal"] = (
                policy_violations_security_total
            )
        if policy_violations_security_audited is not UNSET:
            field_dict["policyViolationsSecurityAudited"] = (
                policy_violations_security_audited
            )
        if policy_violations_security_unaudited is not UNSET:
            field_dict["policyViolationsSecurityUnaudited"] = (
                policy_violations_security_unaudited
            )
        if policy_violations_license_total is not UNSET:
            field_dict["policyViolationsLicenseTotal"] = policy_violations_license_total
        if policy_violations_license_audited is not UNSET:
            field_dict["policyViolationsLicenseAudited"] = (
                policy_violations_license_audited
            )
        if policy_violations_license_unaudited is not UNSET:
            field_dict["policyViolationsLicenseUnaudited"] = (
                policy_violations_license_unaudited
            )
        if policy_violations_operational_total is not UNSET:
            field_dict["policyViolationsOperationalTotal"] = (
                policy_violations_operational_total
            )
        if policy_violations_operational_audited is not UNSET:
            field_dict["policyViolationsOperationalAudited"] = (
                policy_violations_operational_audited
            )
        if policy_violations_operational_unaudited is not UNSET:
            field_dict["policyViolationsOperationalUnaudited"] = (
                policy_violations_operational_unaudited
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        critical = d.pop("critical")

        high = d.pop("high")

        medium = d.pop("medium")

        low = d.pop("low")

        first_occurrence = d.pop("firstOccurrence")

        last_occurrence = d.pop("lastOccurrence")

        unassigned = d.pop("unassigned", UNSET)

        vulnerabilities = d.pop("vulnerabilities", UNSET)

        projects = d.pop("projects", UNSET)

        vulnerable_projects = d.pop("vulnerableProjects", UNSET)

        components = d.pop("components", UNSET)

        vulnerable_components = d.pop("vulnerableComponents", UNSET)

        suppressed = d.pop("suppressed", UNSET)

        findings_total = d.pop("findingsTotal", UNSET)

        findings_audited = d.pop("findingsAudited", UNSET)

        findings_unaudited = d.pop("findingsUnaudited", UNSET)

        inherited_risk_score = d.pop("inheritedRiskScore", UNSET)

        policy_violations_fail = d.pop("policyViolationsFail", UNSET)

        policy_violations_warn = d.pop("policyViolationsWarn", UNSET)

        policy_violations_info = d.pop("policyViolationsInfo", UNSET)

        policy_violations_total = d.pop("policyViolationsTotal", UNSET)

        policy_violations_audited = d.pop("policyViolationsAudited", UNSET)

        policy_violations_unaudited = d.pop("policyViolationsUnaudited", UNSET)

        policy_violations_security_total = d.pop("policyViolationsSecurityTotal", UNSET)

        policy_violations_security_audited = d.pop(
            "policyViolationsSecurityAudited", UNSET
        )

        policy_violations_security_unaudited = d.pop(
            "policyViolationsSecurityUnaudited", UNSET
        )

        policy_violations_license_total = d.pop("policyViolationsLicenseTotal", UNSET)

        policy_violations_license_audited = d.pop(
            "policyViolationsLicenseAudited", UNSET
        )

        policy_violations_license_unaudited = d.pop(
            "policyViolationsLicenseUnaudited", UNSET
        )

        policy_violations_operational_total = d.pop(
            "policyViolationsOperationalTotal", UNSET
        )

        policy_violations_operational_audited = d.pop(
            "policyViolationsOperationalAudited", UNSET
        )

        policy_violations_operational_unaudited = d.pop(
            "policyViolationsOperationalUnaudited", UNSET
        )

        portfolio_metrics = cls(
            critical=critical,
            high=high,
            medium=medium,
            low=low,
            first_occurrence=first_occurrence,
            last_occurrence=last_occurrence,
            unassigned=unassigned,
            vulnerabilities=vulnerabilities,
            projects=projects,
            vulnerable_projects=vulnerable_projects,
            components=components,
            vulnerable_components=vulnerable_components,
            suppressed=suppressed,
            findings_total=findings_total,
            findings_audited=findings_audited,
            findings_unaudited=findings_unaudited,
            inherited_risk_score=inherited_risk_score,
            policy_violations_fail=policy_violations_fail,
            policy_violations_warn=policy_violations_warn,
            policy_violations_info=policy_violations_info,
            policy_violations_total=policy_violations_total,
            policy_violations_audited=policy_violations_audited,
            policy_violations_unaudited=policy_violations_unaudited,
            policy_violations_security_total=policy_violations_security_total,
            policy_violations_security_audited=policy_violations_security_audited,
            policy_violations_security_unaudited=policy_violations_security_unaudited,
            policy_violations_license_total=policy_violations_license_total,
            policy_violations_license_audited=policy_violations_license_audited,
            policy_violations_license_unaudited=policy_violations_license_unaudited,
            policy_violations_operational_total=policy_violations_operational_total,
            policy_violations_operational_audited=policy_violations_operational_audited,
            policy_violations_operational_unaudited=policy_violations_operational_unaudited,
        )

        portfolio_metrics.additional_properties = d
        return portfolio_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="VexSubmitRequest")


@_attrs_define
class VexSubmitRequest:
    """
    Attributes:
        project (str):
        project_name (str):
        project_version (str):
        vex (str):
    """

    project: str
    project_name: str
    project_version: str
    vex: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project

        project_name = self.project_name

        project_version = self.project_version

        vex = self.vex

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project": project,
                "projectName": project_name,
                "projectVersion": project_version,
                "vex": vex,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        project = d.pop("project")

        project_name = d.pop("projectName")

        project_version = d.pop("projectVersion")

        vex = d.pop("vex")

        vex_submit_request = cls(
            project=project,
            project_name=project_name,
            project_version=project_version,
            vex=vex,
        )

        vex_submit_request.additional_properties = d
        return vex_submit_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.affected_version_attribution_source import (
    AffectedVersionAttributionSource,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="AffectedVersionAttribution")


@_attrs_define
class AffectedVersionAttribution:
    """
    Attributes:
        first_seen (int): UNIX epoch timestamp in milliseconds
        last_seen (int): UNIX epoch timestamp in milliseconds
        source (Union[Unset, AffectedVersionAttributionSource]):
        uuid (Union[Unset, UUID]):
    """

    first_seen: int
    last_seen: int
    source: Union[Unset, AffectedVersionAttributionSource] = UNSET
    uuid: Union[Unset, UUID] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        first_seen = self.first_seen

        last_seen = self.last_seen

        source: Union[Unset, str] = UNSET
        if not isinstance(self.source, Unset):
            source = self.source.value

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "firstSeen": first_seen,
                "lastSeen": last_seen,
            }
        )
        if source is not UNSET:
            field_dict["source"] = source
        if uuid is not UNSET:
            field_dict["uuid"] = uuid

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        first_seen = d.pop("firstSeen")

        last_seen = d.pop("lastSeen")

        _source = d.pop("source", UNSET)
        source: Union[Unset, AffectedVersionAttributionSource]
        if isinstance(_source, Unset):
            source = UNSET
        else:
            source = AffectedVersionAttributionSource(_source)

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        affected_version_attribution = cls(
            first_seen=first_seen,
            last_seen=last_seen,
            source=source,
            uuid=uuid,
        )

        affected_version_attribution.additional_properties = d
        return affected_version_attribution

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PolicyConditionOperator(str, Enum):
    CONTAINS_ALL = "CONTAINS_ALL"
    CONTAINS_ANY = "CONTAINS_ANY"
    IS = "IS"
    IS_NOT = "IS_NOT"
    MATCHES = "MATCHES"
    NO_MATCH = "NO_MATCH"
    NUMERIC_EQUAL = "NUMERIC_EQUAL"
    NUMERIC_GREATER_THAN = "NUMERIC_GREATER_THAN"
    NUMERIC_GREATER_THAN_OR_EQUAL = "NUMERIC_GREATER_THAN_OR_EQUAL"
    NUMERIC_LESSER_THAN_OR_EQUAL = "NUMERIC_LESSER_THAN_OR_EQUAL"
    NUMERIC_LESS_THAN = "NUMERIC_LESS_THAN"
    NUMERIC_NOT_EQUAL = "NUMERIC_NOT_EQUAL"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ExternalReferenceType(str, Enum):
    ADVERSARY_MODEL = "adversary-model"
    ADVISORIES = "advisories"
    ATTESTATION = "attestation"
    BOM = "bom"
    BUILD_META = "build-meta"
    BUILD_SYSTEM = "build-system"
    CERTIFICATION_REPORT = "certification-report"
    CHAT = "chat"
    CODIFIED_INFRASTRUCTURE = "codified-infrastructure"
    COMPONENT_ANALYSIS_REPORT = "component-analysis-report"
    CONFIGURATION = "configuration"
    DIGITAL_SIGNATURE = "digital-signature"
    DISTRIBUTION = "distribution"
    DISTRIBUTION_INTAKE = "distribution-intake"
    DOCUMENTATION = "documentation"
    DYNAMIC_ANALYSIS_REPORT = "dynamic-analysis-report"
    ELECTRONIC_SIGNATURE = "electronic-signature"
    EVIDENCE = "evidence"
    EXPLOITABILITY_STATEMENT = "exploitability-statement"
    FORMULATION = "formulation"
    ISSUE_TRACKER = "issue-tracker"
    LICENSE = "license"
    LOG = "log"
    MAILING_LIST = "mailing-list"
    MATURITY_REPORT = "maturity-report"
    MODEL_CARD = "model_card"
    OTHER = "other"
    PENTEST_REPORT = "pentest-report"
    QUALITY_METRICS = "quality-metrics"
    RELEASE_NOTES = "release-notes"
    RFC_9116 = "rfc-9116"
    RISK_ASSESSMENT = "risk-assessment"
    RUNTIME_ANALYSIS_REPORT = "runtime-analysis-report"
    SECURITY_CONTACT = "security-contact"
    SOCIAL = "social"
    SOURCE_DISTRIBUTION = "source-distribution"
    STATIC_ANALYSIS_REPORT = "static-analysis-report"
    SUPPORT = "support"
    THREAT_MODEL = "threat-model"
    VCS = "vcs"
    VULNERABILITY_ASSERTION = "vulnerability-assertion"
    WEBSITE = "website"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.license_group import LicenseGroup


T = TypeVar("T", bound="License")


@_attrs_define
class License:
    """
    Attributes:
        uuid (UUID):
        name (str):
        license_id (str):
        license_groups (Union[Unset, list['LicenseGroup']]):
        license_text (Union[Unset, str]):
        standard_license_template (Union[Unset, str]):
        standard_license_header (Union[Unset, str]):
        license_comments (Union[Unset, str]):
        is_osi_approved (Union[Unset, bool]):
        is_fsf_libre (Union[Unset, bool]):
        is_deprecated_license_id (Union[Unset, bool]):
        is_custom_license (Union[Unset, bool]):
        see_also (Union[Unset, list[str]]):
    """

    uuid: UUID
    name: str
    license_id: str
    license_groups: Union[Unset, list["LicenseGroup"]] = UNSET
    license_text: Union[Unset, str] = UNSET
    standard_license_template: Union[Unset, str] = UNSET
    standard_license_header: Union[Unset, str] = UNSET
    license_comments: Union[Unset, str] = UNSET
    is_osi_approved: Union[Unset, bool] = UNSET
    is_fsf_libre: Union[Unset, bool] = UNSET
    is_deprecated_license_id: Union[Unset, bool] = UNSET
    is_custom_license: Union[Unset, bool] = UNSET
    see_also: Union[Unset, list[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        name = self.name

        license_id = self.license_id

        license_groups: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.license_groups, Unset):
            license_groups = []
            for license_groups_item_data in self.license_groups:
                license_groups_item = license_groups_item_data.to_dict()
                license_groups.append(license_groups_item)

        license_text = self.license_text

        standard_license_template = self.standard_license_template

        standard_license_header = self.standard_license_header

        license_comments = self.license_comments

        is_osi_approved = self.is_osi_approved

        is_fsf_libre = self.is_fsf_libre

        is_deprecated_license_id = self.is_deprecated_license_id

        is_custom_license = self.is_custom_license

        see_also: Union[Unset, list[str]] = UNSET
        if not isinstance(self.see_also, Unset):
            see_also = self.see_also

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "name": name,
                "licenseId": license_id,
            }
        )
        if license_groups is not UNSET:
            field_dict["licenseGroups"] = license_groups
        if license_text is not UNSET:
            field_dict["licenseText"] = license_text
        if standard_license_template is not UNSET:
            field_dict["standardLicenseTemplate"] = standard_license_template
        if standard_license_header is not UNSET:
            field_dict["standardLicenseHeader"] = standard_license_header
        if license_comments is not UNSET:
            field_dict["licenseComments"] = license_comments
        if is_osi_approved is not UNSET:
            field_dict["isOsiApproved"] = is_osi_approved
        if is_fsf_libre is not UNSET:
            field_dict["isFsfLibre"] = is_fsf_libre
        if is_deprecated_license_id is not UNSET:
            field_dict["isDeprecatedLicenseId"] = is_deprecated_license_id
        if is_custom_license is not UNSET:
            field_dict["isCustomLicense"] = is_custom_license
        if see_also is not UNSET:
            field_dict["seeAlso"] = see_also

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.license_group import LicenseGroup

        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        name = d.pop("name")

        license_id = d.pop("licenseId")

        license_groups = []
        _license_groups = d.pop("licenseGroups", UNSET)
        for license_groups_item_data in _license_groups or []:
            license_groups_item = LicenseGroup.from_dict(license_groups_item_data)

            license_groups.append(license_groups_item)

        license_text = d.pop("licenseText", UNSET)

        standard_license_template = d.pop("standardLicenseTemplate", UNSET)

        standard_license_header = d.pop("standardLicenseHeader", UNSET)

        license_comments = d.pop("licenseComments", UNSET)

        is_osi_approved = d.pop("isOsiApproved", UNSET)

        is_fsf_libre = d.pop("isFsfLibre", UNSET)

        is_deprecated_license_id = d.pop("isDeprecatedLicenseId", UNSET)

        is_custom_license = d.pop("isCustomLicense", UNSET)

        see_also = cast(list[str], d.pop("seeAlso", UNSET))

        license_ = cls(
            uuid=uuid,
            name=name,
            license_id=license_id,
            license_groups=license_groups,
            license_text=license_text,
            standard_license_template=standard_license_template,
            standard_license_header=standard_license_header,
            license_comments=license_comments,
            is_osi_approved=is_osi_approved,
            is_fsf_libre=is_fsf_libre,
            is_deprecated_license_id=is_deprecated_license_id,
            is_custom_license=is_custom_license,
            see_also=see_also,
        )

        license_.additional_properties = d
        return license_

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission
    from ..models.team import Team


T = TypeVar("T", bound="ManagedUser")


@_attrs_define
class ManagedUser:
    """
    Attributes:
        username (str):
        last_password_change (int): UNIX epoch timestamp in milliseconds
        fullname (Union[Unset, str]):
        email (Union[Unset, str]):
        suspended (Union[Unset, bool]):
        force_password_change (Union[Unset, bool]):
        non_expiry_password (Union[Unset, bool]):
        teams (Union[Unset, list['Team']]):
        permissions (Union[Unset, list['Permission']]):
        new_password (Union[Unset, str]):
        confirm_password (Union[Unset, str]):
    """

    username: str
    last_password_change: int
    fullname: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    suspended: Union[Unset, bool] = UNSET
    force_password_change: Union[Unset, bool] = UNSET
    non_expiry_password: Union[Unset, bool] = UNSET
    teams: Union[Unset, list["Team"]] = UNSET
    permissions: Union[Unset, list["Permission"]] = UNSET
    new_password: Union[Unset, str] = UNSET
    confirm_password: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        username = self.username

        last_password_change = self.last_password_change

        fullname = self.fullname

        email = self.email

        suspended = self.suspended

        force_password_change = self.force_password_change

        non_expiry_password = self.non_expiry_password

        teams: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()
                teams.append(teams_item)

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        new_password = self.new_password

        confirm_password = self.confirm_password

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "username": username,
                "lastPasswordChange": last_password_change,
            }
        )
        if fullname is not UNSET:
            field_dict["fullname"] = fullname
        if email is not UNSET:
            field_dict["email"] = email
        if suspended is not UNSET:
            field_dict["suspended"] = suspended
        if force_password_change is not UNSET:
            field_dict["forcePasswordChange"] = force_password_change
        if non_expiry_password is not UNSET:
            field_dict["nonExpiryPassword"] = non_expiry_password
        if teams is not UNSET:
            field_dict["teams"] = teams
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if new_password is not UNSET:
            field_dict["newPassword"] = new_password
        if confirm_password is not UNSET:
            field_dict["confirmPassword"] = confirm_password

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.permission import Permission
        from ..models.team import Team

        d = dict(src_dict)
        username = d.pop("username")

        last_password_change = d.pop("lastPasswordChange")

        fullname = d.pop("fullname", UNSET)

        email = d.pop("email", UNSET)

        suspended = d.pop("suspended", UNSET)

        force_password_change = d.pop("forcePasswordChange", UNSET)

        non_expiry_password = d.pop("nonExpiryPassword", UNSET)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = Team.from_dict(teams_item_data)

            teams.append(teams_item)

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        new_password = d.pop("newPassword", UNSET)

        confirm_password = d.pop("confirmPassword", UNSET)

        managed_user = cls(
            username=username,
            last_password_change=last_password_change,
            fullname=fullname,
            email=email,
            suspended=suspended,
            force_password_change=force_password_change,
            non_expiry_password=non_expiry_password,
            teams=teams,
            permissions=permissions,
            new_password=new_password,
            confirm_password=confirm_password,
        )

        managed_user.additional_properties = d
        return managed_user

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.policy_condition_operator import PolicyConditionOperator
from ..models.policy_condition_subject import PolicyConditionSubject
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.policy import Policy


T = TypeVar("T", bound="PolicyCondition")


@_attrs_define
class PolicyCondition:
    """
    Attributes:
        operator (PolicyConditionOperator):
        subject (PolicyConditionSubject):
        value (str):
        uuid (UUID):
        policy (Union[Unset, Policy]):
    """

    operator: PolicyConditionOperator
    subject: PolicyConditionSubject
    value: str
    uuid: UUID
    policy: Union[Unset, "Policy"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        operator = self.operator.value

        subject = self.subject.value

        value = self.value

        uuid = str(self.uuid)

        policy: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.policy, Unset):
            policy = self.policy.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "operator": operator,
                "subject": subject,
                "value": value,
                "uuid": uuid,
            }
        )
        if policy is not UNSET:
            field_dict["policy"] = policy

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.policy import Policy

        d = dict(src_dict)
        operator = PolicyConditionOperator(d.pop("operator"))

        subject = PolicyConditionSubject(d.pop("subject"))

        value = d.pop("value")

        uuid = UUID(d.pop("uuid"))

        _policy = d.pop("policy", UNSET)
        policy: Union[Unset, Policy]
        if isinstance(_policy, Unset):
            policy = UNSET
        else:
            policy = Policy.from_dict(_policy)

        policy_condition = cls(
            operator=operator,
            subject=subject,
            value=value,
            uuid=uuid,
            policy=policy,
        )

        policy_condition.additional_properties = d
        return policy_condition

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union, cast
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_all_services_sort_order import GetAllServicesSortOrder
from ...models.service_component import ServiceComponent
from ...types import UNSET, Response, Unset


def _get_kwargs(
    uuid: UUID,
    *,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllServicesSortOrder] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["pageNumber"] = page_number

    params["pageSize"] = page_size

    params["offset"] = offset

    params["limit"] = limit

    params["sortName"] = sort_name

    json_sort_order: Union[Unset, str] = UNSET
    if not isinstance(sort_order, Unset):
        json_sort_order = sort_order.value

    params["sortOrder"] = json_sort_order

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/service/project/{uuid}".format(
            uuid=uuid,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, list["ServiceComponent"]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ServiceComponent.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403

    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, list["ServiceComponent"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllServicesSortOrder] = UNSET,
) -> Response[Union[Any, list["ServiceComponent"]]]:
    """Returns a list of all services for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllServicesSortOrder]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['ServiceComponent']]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllServicesSortOrder] = UNSET,
) -> Optional[Union[Any, list["ServiceComponent"]]]:
    """Returns a list of all services for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllServicesSortOrder]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['ServiceComponent']]
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllServicesSortOrder] = UNSET,
) -> Response[Union[Any, list["ServiceComponent"]]]:
    """Returns a list of all services for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllServicesSortOrder]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['ServiceComponent']]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllServicesSortOrder] = UNSET,
) -> Optional[Union[Any, list["ServiceComponent"]]]:
    """Returns a list of all services for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllServicesSortOrder]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['ServiceComponent']]
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
            page_number=page_number,
            page_size=page_size,
            offset=offset,
            limit=limit,
            sort_name=sort_name,
            sort_order=sort_order,
        )
    ).parsed
